/* Dia -- an diagram creation/manipulation program
 * Copyright (C) 1998 Alexander Larsson
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
#ifndef SELECT_H
#define SELECT_H

#include "diagram.h"
#include "display.h"

typedef enum {
  SELECT_REPLACE, 
  SELECT_UNION,
  SELECT_INTERSECTION,
  SELECT_REMOVE,
  SELECT_INVERT
} SelectionStyle;

extern SelectionStyle selection_style;

void select_all_callback(GtkAction *action);
void select_none_callback(GtkAction *action);
void select_invert_callback(GtkAction *action);
void select_connected_callback(GtkAction *action);
void select_transitive_callback(GtkAction *action);
void select_same_type_callback(GtkAction *action);
void select_style_callback(GtkAction *action, GtkRadioAction *current, gpointer user_data);

#endif
